#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the Windows 9x/Me Registry (CREG) back-end."""

import unittest

from dfwinreg import errors
from dfwinreg import creg

from tests import test_lib


class FakePyCREGKey(object):
  """Fake pycreg key for testing."""

  def __init__(self):
    """Initializes a fake pycreg key."""
    super(FakePyCREGKey, self).__init__()
    self.number_of_sub_keys = 1

  # pylint: disable=invalid-name,redundant-returns-doc,unused-argument

  def get_last_written_time_as_integer(self):
    """Retrieves the last written time as an integer.

    Returns:
      int: last written time, which will be 0 for testing.
    """
    return 0

  def get_sub_key(self, sub_key_index):
    """Retrieves a specific sub key.

    Returns:
      pycreg.key: sub key, which will be None for testing.
    """
    return None


class FakePyCREGValue(object):
  """Fake pycreg value for testing.

  Attributes:
    name (str): name of the value.
    type (str): value type.
  """

  def __init__(self, name='Test', value_type='REG_SZ'):
    """Initializes a fake pycreg value.

    Args:
      name (Optional[str]): name of the value.
      value_type (Optional[str]): value type.
    """
    super(FakePyCREGValue, self).__init__()
    self.name = name
    self.type = value_type

  # pylint: disable=missing-raises-doc

  @property
  def data(self):
    """bytes: value data."""
    raise IOError('raised for testing purposes.')


class CREGWinRegistryFileTest(test_lib.BaseTestCase):
  """Tests for the CREG Windows Registry file."""

  def testOpenClose(self):
    """Tests the Open and Close functions."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)
      registry_file.Close()

  def testGetRootKey(self):
    """Tests the GetRootKey function."""
    dat_test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(dat_test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(dat_test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetRootKey()
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.path, '\\')

      finally:
        registry_file.Close()

  def testGetKeyByPath(self):
    """Tests the GetKeyByPath function."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.path, key_path)

        key_path = '\\Software'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.path, key_path)

        key_path = '\\Bogus'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNone(registry_key)

        key_path = 'Bogus'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNone(registry_key)

      finally:
        registry_file.Close()

  def testRecurseKeys(self):
    """Tests the RecurseKeys function."""
    dat_test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(dat_test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(dat_test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_keys = list(registry_file.RecurseKeys())

      finally:
        registry_file.Close()

    self.assertEqual(len(registry_keys), 788)


class CREGWinRegistryKeyTest(test_lib.BaseTestCase):
  """Tests for the CREG Windows Registry key."""

  # pylint: disable=protected-access

  def testProperties(self):
    """Tests the properties functions."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\Software'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertIsNone(registry_key.class_name)
        self.assertEqual(registry_key.name, 'Software')
        self.assertEqual(registry_key.number_of_subkeys, 1)
        self.assertEqual(registry_key.number_of_values, 0)
        self.assertEqual(registry_key.offset, 27948)
        self.assertEqual(registry_key.path, key_path)

        self.assertIsNotNone(registry_key.last_written_time)

        date_time_string = registry_key.last_written_time.CopyToDateTimeString()
        self.assertEqual(date_time_string, 'Not set')

        registry_key._pycreg_key = FakePyCREGKey()
        self.assertIsNotNone(registry_key.last_written_time)

        date_time_string = registry_key.last_written_time.CopyToDateTimeString()
        self.assertEqual(date_time_string, 'Not set')

      finally:
        registry_file.Close()

  def testGetSubkeyByIndex(self):
    """Tests the GetSubkeyByIndex function."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile(
        key_path_prefix='HKEY_CURRENT_USER')

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetRootKey()

        key_name = '.DEFAULT'
        sub_registry_key = registry_key.GetSubkeyByIndex(0)
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, key_name)

        expected_key_path = 'HKEY_CURRENT_USER\\.DEFAULT'
        self.assertEqual(sub_registry_key.path, expected_key_path)

        with self.assertRaises(IndexError):
          registry_key.GetSubkeyByIndex(-1)

      finally:
        registry_file.Close()

  def testGetSubkeyByName(self):
    """Tests the GetSubkeyByName function."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile(
        key_path_prefix='HKEY_CURRENT_USER')

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetRootKey()

        key_name = 'Software'
        sub_registry_key = registry_key.GetSubkeyByName(key_name)
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, key_name)

        expected_key_path = 'HKEY_CURRENT_USER\\Software'
        self.assertEqual(sub_registry_key.path, expected_key_path)

        key_name = 'Bogus'
        sub_registry_key = registry_key.GetSubkeyByName(key_name)
        self.assertIsNone(sub_registry_key)

      finally:
        registry_file.Close()

  def testGetSubkeyByPath(self):
    """Tests the GetSubkeyByPath function."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile(
        key_path_prefix='HKEY_CURRENT_USER')

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetRootKey()

        key_path = 'Software\\Microsoft'
        sub_registry_key = registry_key.GetSubkeyByPath(key_path)
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, 'Microsoft')

        expected_key_path = 'HKEY_CURRENT_USER\\Software\\Microsoft'
        self.assertEqual(sub_registry_key.path, expected_key_path)

        key_path = 'Software\\Bogus'
        sub_registry_key = registry_key.GetSubkeyByPath(key_path)
        self.assertIsNone(sub_registry_key)

      finally:
        registry_file.Close()

  def testGetSubkeys(self):
    """Tests the GetSubkeys function."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\Software'
        registry_key = registry_file.GetKeyByPath(key_path)

        sub_registry_keys = list(registry_key.GetSubkeys())
        self.assertEqual(len(sub_registry_keys), 1)

      finally:
        registry_file.Close()

  def testGetValueByName(self):
    """Tests the GetValueByName function."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetKeyByPath(
            '\\.DEFAULT\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer'
            '\\MountPoints\\A\\_Autorun')

        value_name = 'LastUpdate'
        registry_value = registry_key.GetValueByName(value_name)
        self.assertIsNotNone(registry_value)
        self.assertEqual(registry_value.name, value_name)

        value_name = 'Bogus'
        registry_value = registry_key.GetValueByName(value_name)
        self.assertIsNone(registry_value)

        # Test retrieving the default (or nameless) value.
        registry_key = registry_file.GetKeyByPath(
            '\\.DEFAULT\\AppEvents\\EventLabels\\.Default')

        registry_value = registry_key.GetValueByName('')
        self.assertIsNotNone(registry_value)
        self.assertIsNone(registry_value.name)

      finally:
        registry_file.Close()

  def testGetValues(self):
    """Tests the GetValues function."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = (
            '\\.DEFAULT\\Software\\Microsoft\\Windows\\CurrentVersion'
            '\\Explorer\\MountPoints\\A\\_Autorun')
        registry_key = registry_file.GetKeyByPath(key_path)

        values = list(registry_key.GetValues())
        self.assertEqual(len(values), 3)

      finally:
        registry_file.Close()

  def testRecurseKeys(self):
    """Tests the RecurseKeys function."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\Software'
        registry_key = registry_file.GetKeyByPath(key_path)
        registry_keys = list(registry_key.RecurseKeys())

      finally:
        registry_file.Close()

    self.assertEqual(len(registry_keys), 6)


class CREGWinRegistryValueTest(test_lib.BaseTestCase):
  """Tests for the CREG Windows Registry value."""

  # pylint: disable=protected-access

  _EXPECTED_BINARY_DATA_VALUE = bytes(bytearray([
      0x04, 0x00, 0x00, 0x00, 0x1f, 0x30, 0x8c, 0xd3, 0x01, 0x00, 0x00, 0x00,
      0x0d, 0x00, 0x00, 0x00, 0x0d, 0x00, 0x00, 0x00, 0x12, 0x00, 0x00, 0x00,
      0x12, 0x00, 0x00, 0x00, 0xf4, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xbc, 0x02, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x41, 0x72, 0x69, 0x61,
      0x6c, 0x00, 0x17, 0x01, 0x50, 0x13, 0x78, 0x45, 0x00, 0x00, 0xc8, 0x45,
      0x00, 0x00, 0x48, 0x7e, 0x00, 0x00, 0x30, 0x7e, 0x63, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x01, 0x00, 0x0f, 0x00, 0x00, 0x00, 0x0f, 0x00, 0x00, 0x00,
      0xf7, 0xff, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x90, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x4d, 0x53, 0x20, 0x53, 0x61, 0x6e, 0x73, 0x20,
      0x53, 0x65, 0x72, 0x69, 0x66, 0x00, 0xc8, 0x45, 0x00, 0x00, 0x48, 0x7e,
      0x00, 0x00, 0x30, 0x7e, 0x63, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00,
      0x12, 0x00, 0x00, 0x00, 0x12, 0x00, 0x00, 0x00, 0xf5, 0xff, 0xff, 0xff,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x90, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x4d, 0x53, 0x20, 0x53, 0x61, 0x6e, 0x73, 0x20, 0x53, 0x65, 0x72, 0x69,
      0x66, 0x00, 0xc8, 0x45, 0x00, 0x00, 0x48, 0x7e, 0x00, 0x00, 0x30, 0x7e,
      0x63, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0xf5, 0xff, 0xff, 0xff,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x90, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x4d, 0x53, 0x20, 0x53, 0x61, 0x6e, 0x73, 0x20, 0x53, 0x65, 0x72, 0x69,
      0x66, 0x00, 0xc8, 0x45, 0x00, 0x00, 0x48, 0x7e, 0x00, 0x00, 0x30, 0x7e,
      0x63, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0xf5, 0xff, 0xff, 0xff,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x90, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x4d, 0x53, 0x20, 0x53, 0x61, 0x6e, 0x73, 0x20, 0x53, 0x65, 0x72, 0x69,
      0x66, 0x00, 0xc8, 0x45, 0x00, 0x00, 0x48, 0x7e, 0x00, 0x00, 0x30, 0x7e,
      0x63, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0xf8, 0xff, 0xff, 0xff,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x90, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x4d, 0x53, 0x20, 0x53, 0x61, 0x6e, 0x73, 0x20, 0x53, 0x65, 0x72, 0x69,
      0x66, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe1, 0xe0, 0xd2, 0x02,
      0x42, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x8d, 0x89, 0x61, 0x00,
      0xc2, 0xbf, 0xa5, 0x02, 0xff, 0xff, 0xff, 0x02, 0x00, 0x00, 0x00, 0x02,
      0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x02, 0xe1, 0xe0, 0xd2, 0x02,
      0xc2, 0xbf, 0xa5, 0x02, 0xc2, 0xbf, 0xa5, 0x02, 0xe1, 0xe0, 0xd2, 0x02,
      0x8d, 0x89, 0x61, 0x02, 0xff, 0xff, 0xff, 0x02, 0xc2, 0xbf, 0xa5, 0x02,
      0x8d, 0x89, 0x61, 0x02, 0x8d, 0x89, 0x61, 0x02, 0x00, 0x00, 0x00, 0x02,
      0xe1, 0xe0, 0xd2, 0x02, 0xe1, 0xe0, 0xd2, 0x02, 0x00, 0x00, 0x00, 0x02,
      0xc2, 0xbf, 0xa5, 0x02, 0x80, 0x00, 0x00, 0x02, 0xe1, 0xe0, 0xd2, 0x02,
      0xba, 0xb7, 0x9a, 0x00, 0x00, 0x00, 0xff, 0x00, 0x80, 0x00, 0x00, 0x00,
      0x8d, 0x89, 0x61, 0x00]))

  def testProperties(self):
    """Tests the properties functions on a USER.DAT file."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetKeyByPath(
            '\\.DEFAULT\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer'
            '\\MountPoints\\A\\_Autorun')
        value_name = 'LastUpdate'
        registry_value = registry_key.GetValueByName(value_name)
        expected_data = b'\x8e\x7d\x02\x00'

        self.assertIsNotNone(registry_value)
        self.assertEqual(registry_value.data_type, 4)
        self.assertEqual(registry_value.data_type_string, 'REG_DWORD_LE')
        self.assertEqual(registry_value.GetDataAsObject(), 163214)
        self.assertEqual(registry_value.name, value_name)
        self.assertEqual(registry_value.offset, 90258)
        self.assertEqual(registry_value.data, expected_data)

        registry_key = registry_file.GetKeyByPath(
            '\\.DEFAULT\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer'
            '\\Shell Folders')
        value_name = 'Personal'
        registry_value = registry_key.GetValueByName(value_name)
        expected_data = (
            b'\x43\x3a\x5c\x4d\x79\x20\x44\x6f\x63\x75\x6d\x65\x6e\x74\x73')

        self.assertIsNotNone(registry_value)
        self.assertEqual(registry_value.data_type, 1)
        self.assertEqual(registry_value.data_type_string, 'REG_SZ')
        self.assertEqual(registry_value.GetDataAsObject(), 'C:\\My Documents')
        self.assertEqual(registry_value.name, value_name)
        self.assertEqual(registry_value.offset, 57547)
        self.assertEqual(registry_value.data, expected_data)

        registry_key = registry_file.GetKeyByPath(
            '\\.DEFAULT\\Control Panel\\Appearance\\Schemes')
        value_name = 'Brick'
        registry_value = registry_key.GetValueByName(value_name)
        expected_data = self._EXPECTED_BINARY_DATA_VALUE

        self.assertIsNotNone(registry_value)
        self.assertEqual(registry_value.data_type, 3)
        self.assertEqual(registry_value.data_type_string, 'REG_BINARY')
        self.assertEqual(registry_value.GetDataAsObject(), expected_data)
        self.assertEqual(registry_value.name, value_name)
        self.assertEqual(registry_value.offset, 30271)
        self.assertEqual(registry_value.data, expected_data)

        registry_value._pycreg_value = FakePyCREGValue()
        with self.assertRaises(errors.WinRegistryValueError):
          _ = registry_value.data

      finally:
        registry_file.Close()

  def testGetDataAsObject(self):
    """Tests the GetDataAsObject function."""
    test_path = self._GetTestFilePath(['USER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = creg.CREGWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetKeyByPath(
            '\\.DEFAULT\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer'
            '\\MountPoints\\A\\_Autorun')
        registry_value = registry_key.GetValueByName('LastUpdate')

        data_object = registry_value.GetDataAsObject()
        self.assertEqual(data_object, 163214)

        registry_value._pycreg_value = FakePyCREGValue(value_type='REG_SZ')
        with self.assertRaises(errors.WinRegistryValueError):
          registry_value.GetDataAsObject()

        registry_value._pycreg_value = FakePyCREGValue(
            value_type='REG_DWORD_LE')
        with self.assertRaises(errors.WinRegistryValueError):
          registry_value.GetDataAsObject()

        registry_value._pycreg_value = FakePyCREGValue(
            value_type='REG_MULTI_SZ')
        with self.assertRaises(errors.WinRegistryValueError):
          registry_value.GetDataAsObject()

      finally:
        registry_file.Close()


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the fake Windows Registry back-end."""

import unittest

from dfwinreg import definitions
from dfwinreg import errors
from dfwinreg import fake

from tests import test_lib


class FakeWinRegTestCase(test_lib.BaseTestCase):
  """The unit test case for fake Windows Registry related."""

  def _OpenFakeRegistryFile(self, key_path_prefix=''):
    """Opens a fake Windows Registry file.

    Args:
      key_path_prefix (str): Windows Registry key path prefix.

    Returns:
      FakeWinRegistryFileTest: fake Windows Registry file.
    """
    registry_file = fake.FakeWinRegistryFile(
        key_path_prefix=key_path_prefix)

    software_key = fake.FakeWinRegistryKey('Software')
    registry_file.AddKeyByPath('\\', software_key)

    registry_file.Open(None)
    return registry_file


class FakeWinRegistryFileTest(FakeWinRegTestCase):
  """Tests for a fake Windows Registry file."""

  def testAddKeyByPath(self):
    """Tests the AddKeyByPath function."""
    registry_file = fake.FakeWinRegistryFile()

    software_key = fake.FakeWinRegistryKey('Software')
    registry_file.AddKeyByPath('\\', software_key)

    test_key = fake.FakeWinRegistryKey('Key')
    registry_file.AddKeyByPath('\\Test\\Path', test_key)

    test_key = fake.FakeWinRegistryKey('More')
    registry_file.AddKeyByPath('\\Test\\Path\\Key', test_key)

    with self.assertRaises(KeyError):
      registry_file.AddKeyByPath('\\', software_key)

    with self.assertRaises(ValueError):
      registry_file.AddKeyByPath('Test', software_key)

  def testOpenClose(self):
    """Tests the Open and Close functions."""
    registry_file = self._OpenFakeRegistryFile()
    registry_file.Close()

  def testGetKeyByPath(self):
    """Tests the GetKeyByPath function."""
    registry_file = fake.FakeWinRegistryFile()

    registry_key = registry_file.GetKeyByPath('\\')
    self.assertIsNone(registry_key)

    registry_file = self._OpenFakeRegistryFile(
        key_path_prefix='HKEY_LOCAL_MACHINE')

    test_key = fake.FakeWinRegistryKey('Key')
    registry_file.AddKeyByPath('\\Test\\Path', test_key)

    # Test root key without prefix.
    key_path = '\\'
    registry_key = registry_file.GetKeyByPath(key_path)
    self.assertIsNotNone(registry_key)
    self.assertEqual(registry_key.path, key_path)

    # Test root key with prefix.
    key_path = 'HKEY_LOCAL_MACHINE\\'
    registry_key = registry_file.GetKeyByPath(key_path)
    self.assertIsNotNone(registry_key)
    self.assertEqual(registry_key.path, '\\')

    # Test key without prefix.
    key_path = '\\Software'
    registry_key = registry_file.GetKeyByPath(key_path)
    self.assertIsNotNone(registry_key)
    self.assertEqual(registry_key.path, key_path)

    # Test key with prefix.
    key_path = 'HKEY_LOCAL_MACHINE\\Software'
    registry_key = registry_file.GetKeyByPath(key_path)
    self.assertIsNotNone(registry_key)
    self.assertEqual(registry_key.path, '\\Software')

    # Test key with some depth.
    key_path = '\\Test\\Path\\Key'
    registry_key = registry_file.GetKeyByPath(key_path)
    self.assertIsNotNone(registry_key)
    self.assertEqual(registry_key.path, key_path)

    # Test non-existing keys.
    key_path = '\\Bogus'
    registry_key = registry_file.GetKeyByPath(key_path)
    self.assertIsNone(registry_key)

    key_path = '\\Test\\Path\\Bogus'
    registry_key = registry_file.GetKeyByPath(key_path)
    self.assertIsNone(registry_key)

    key_path = 'Bogus'
    registry_key = registry_file.GetKeyByPath(key_path)
    self.assertIsNone(registry_key)

    registry_file.Close()

  def testGetRootKey(self):
    """Tests the GetRootKey function."""
    registry_file = self._OpenFakeRegistryFile()

    registry_key = registry_file.GetRootKey()
    self.assertIsNotNone(registry_key)
    self.assertEqual(registry_key.path, '\\')

    registry_file.Close()

  def testRecurseKeys(self):
    """Tests the RecurseKeys function."""
    registry_file = self._OpenFakeRegistryFile()

    registry_keys = list(registry_file.RecurseKeys())
    registry_file.Close()

    self.assertEqual(len(registry_keys), 2)


class FakeWinRegistryKeyTest(test_lib.BaseTestCase):
  """Tests for a fake Windows Registry key."""

  # pylint: disable=protected-access

  def _CreateTestKey(self):
    """Creates a fake Windows Registry key for testing.

    Returns:
      FakeWinRegistryKey: fake Windows Registry key.
    """
    registry_key = fake.FakeWinRegistryKey(
        'Software', key_path='HKEY_CURRENT_USER\\Software',
        last_written_time=0)

    sub_registry_key = fake.FakeWinRegistryKey(
        'Microsoft', last_written_time=0)
    registry_key.AddSubkey(sub_registry_key.name, sub_registry_key)

    test_registry_key = fake.FakeWinRegistryKey(
        'Internet Explorer', last_written_time=0)
    sub_registry_key.AddSubkey(test_registry_key.name, test_registry_key)

    registry_value = fake.FakeWinRegistryValue('')
    registry_key.AddValue(registry_value)

    return registry_key

  def testProperties(self):
    """Tests the properties."""
    registry_key = self._CreateTestKey()
    self.assertIsNotNone(registry_key)

    self.assertIsNone(registry_key.class_name)
    self.assertIsNotNone(registry_key.last_written_time)
    self.assertEqual(registry_key.last_written_time.timestamp, 0)

    self.assertEqual(registry_key.number_of_subkeys, 1)
    self.assertEqual(registry_key.number_of_values, 1)
    self.assertIsNone(registry_key.offset)

    registry_key._last_written_time = None
    self.assertIsNotNone(registry_key.last_written_time)

    date_time_string = registry_key.last_written_time.CopyToDateTimeString()
    self.assertEqual(date_time_string, 'Not set')

  def testBuildKeyHierarchy(self):
    """Tests the BuildKeyHierarchy function."""
    test_key = fake.FakeWinRegistryKey(
        'Microsoft', key_path='HKEY_CURRENT_USER\\Software\\Microsoft',
        last_written_time=0)

    test_value = fake.FakeWinRegistryValue('')

    self.assertIsNotNone(test_key)
    self.assertIsNotNone(test_value)

    # Test with subkeys and values.
    registry_key = fake.FakeWinRegistryKey(
        'Software', key_path='HKEY_CURRENT_USER\\Software',
        last_written_time=0,
        subkeys=[test_key], values=[test_value])
    self.assertIsNotNone(registry_key)

    # Test with duplicate subkeys and values.
    registry_key = fake.FakeWinRegistryKey(
        'Software', key_path='HKEY_CURRENT_USER\\Software',
        last_written_time=0,
        subkeys=[test_key, test_key],
        values=[test_value, test_value])
    self.assertIsNotNone(registry_key)

  def testAddSubkey(self):
    """Tests the AddSubkey function."""
    registry_key = fake.FakeWinRegistryKey(
        'Software', key_path='HKEY_CURRENT_USER\\Software',
        last_written_time=0)

    sub_registry_key = fake.FakeWinRegistryKey(
        'Microsoft', key_path='HKEY_CURRENT_USER\\Software\\Microsoft',
        last_written_time=0)

    registry_key.AddSubkey(sub_registry_key.name, sub_registry_key)

    with self.assertRaises(KeyError):
      registry_key.AddSubkey(sub_registry_key.name, sub_registry_key)

  def testAddValue(self):
    """Tests the AddValue function."""
    registry_key = fake.FakeWinRegistryKey(
        'Software', key_path='HKEY_CURRENT_USER\\Software',
        last_written_time=0)

    registry_value = fake.FakeWinRegistryValue('')

    registry_key.AddValue(registry_value)

    with self.assertRaises(KeyError):
      registry_key.AddValue(registry_value)

  def testGetSubkeyByIndex(self):
    """Tests the GetSubkeyByIndex function."""
    registry_key = self._CreateTestKey()

    sub_registry_key = registry_key.GetSubkeyByIndex(0)
    self.assertIsNotNone(sub_registry_key)

    expected_key_path = 'HKEY_CURRENT_USER\\Software\\Microsoft'
    self.assertEqual(sub_registry_key.path, expected_key_path)

    with self.assertRaises(IndexError):
      registry_key.GetSubkeyByIndex(-1)

  def testGetSubkeyByName(self):
    """Tests the GetSubkeyByName function."""
    registry_key = self._CreateTestKey()

    sub_registry_key = registry_key.GetSubkeyByName('Microsoft')
    self.assertIsNotNone(sub_registry_key)

    expected_key_path = 'HKEY_CURRENT_USER\\Software\\Microsoft'
    self.assertEqual(sub_registry_key.path, expected_key_path)

    sub_registry_key = registry_key.GetSubkeyByName('Bogus')
    self.assertIsNone(sub_registry_key)

  def testGetSubkeyByPath(self):
    """Tests the GetSubkeyByPath function."""
    registry_key = self._CreateTestKey()

    key_path = 'Microsoft\\Internet Explorer'
    sub_registry_key = registry_key.GetSubkeyByPath(key_path)
    self.assertIsNotNone(sub_registry_key)
    self.assertEqual(sub_registry_key.name, 'Internet Explorer')

    expected_key_path = (
        'HKEY_CURRENT_USER\\Software\\Microsoft\\Internet Explorer')
    self.assertEqual(sub_registry_key.path, expected_key_path)

    key_path = 'Microsoft\\Bogus'
    sub_registry_key = registry_key.GetSubkeyByPath(key_path)
    self.assertIsNone(sub_registry_key)

  def testGetSubkeys(self):
    """Tests the GetSubkeys function."""
    registry_key = self._CreateTestKey()

    sub_registry_keys = list(registry_key.GetSubkeys())
    self.assertEqual(len(sub_registry_keys), 1)

  def testGetValueByName(self):
    """Tests the GetValueByName function."""
    registry_key = self._CreateTestKey()

    registry_value = registry_key.GetValueByName('')
    self.assertIsNotNone(registry_value)

    registry_value = registry_key.GetValueByName('Bogus')
    self.assertIsNone(registry_value)

  def testGetValues(self):
    """Tests the GetValues function."""
    registry_key = self._CreateTestKey()

    values = list(registry_key.GetValues())
    self.assertEqual(len(values), 1)


class FakeWinRegistryValueTest(test_lib.BaseTestCase):
  """Tests for a fake Windows Registry value."""

  def testProperties(self):
    """Tests the properties."""
    registry_value = fake.FakeWinRegistryValue(
        'MRUListEx', data_type=definitions.REG_BINARY)
    self.assertIsNotNone(registry_value)

    self.assertEqual(registry_value.data, b'')
    self.assertEqual(registry_value.data_type, definitions.REG_BINARY)
    self.assertEqual(registry_value.name, 'MRUListEx')
    self.assertEqual(registry_value.offset, 0)

  def testGetDataAsObject(self):
    """Tests the GetDataAsObject function."""
    registry_value = fake.FakeWinRegistryValue(
        'MRUListEx', data_type=definitions.REG_BINARY)

    value_data = registry_value.GetDataAsObject()
    self.assertIsNone(value_data)

    registry_value = fake.FakeWinRegistryValue(
        'MRUListEx', data=b'DATA', data_type=definitions.REG_BINARY)

    value_data = registry_value.GetDataAsObject()
    self.assertEqual(value_data, b'DATA')

    data = 'ValueData'.encode('utf-16-le')
    registry_value = fake.FakeWinRegistryValue(
        'MRU', data=data, data_type=definitions.REG_SZ)

    value_data = registry_value.GetDataAsObject()
    self.assertEqual(value_data, 'ValueData')

    data = '\xed\x44'
    registry_value = fake.FakeWinRegistryValue(
        'MRU', data=data, data_type=definitions.REG_SZ)

    with self.assertRaises(errors.WinRegistryValueError):
      registry_value.GetDataAsObject()

    registry_value = fake.FakeWinRegistryValue(
        'Count', data=b'\x11\x22\x33\x44', data_type=definitions.REG_DWORD)

    value_data = registry_value.GetDataAsObject()
    self.assertEqual(value_data, 0x44332211)

    registry_value = fake.FakeWinRegistryValue(
        'Count', data=b'\x11\x22\x33\x44',
        data_type=definitions.REG_DWORD_BIG_ENDIAN)

    value_data = registry_value.GetDataAsObject()
    self.assertEqual(value_data, 0x11223344)

    registry_value = fake.FakeWinRegistryValue(
        'Count', data=b'\x88\x77\x66\x55\x44\x33\x22\x11',
        data_type=definitions.REG_QWORD)

    value_data = registry_value.GetDataAsObject()
    self.assertEqual(value_data, 0x1122334455667788)

    # Test REG_MULTI_SZ without additional empty string.
    data = b'\x65\x00\x6e\x00\x2d\x00\x55\x00\x53\x00\x00\x00'
    registry_value = fake.FakeWinRegistryValue(
        'MRU', data=data, data_type=definitions.REG_MULTI_SZ)

    value_data = registry_value.GetDataAsObject()
    self.assertEqual(value_data, ['en-US'])

    # Test REG_MULTI_SZ with additional empty string.
    data = (
        b'\x2e\x00\x62\x00\x6d\x00\x70\x00\x00\x00\x2e\x00\x63\x00\x6f\x00'
        b'\x6e\x00\x74\x00\x61\x00\x63\x00\x74\x00\x00\x00\x2e\x00\x6a\x00'
        b'\x6e\x00\x74\x00\x00\x00\x2e\x00\x6c\x00\x69\x00\x62\x00\x72\x00'
        b'\x61\x00\x72\x00\x79\x00\x2d\x00\x6d\x00\x73\x00\x00\x00\x2e\x00'
        b'\x6c\x00\x6e\x00\x6b\x00\x00\x00\x2e\x00\x72\x00\x74\x00\x66\x00'
        b'\x00\x00\x2e\x00\x74\x00\x78\x00\x74\x00\x00\x00\x2e\x00\x7a\x00'
        b'\x69\x00\x70\x00\x00\x00\x46\x00\x6f\x00\x6c\x00\x64\x00\x65\x00'
        b'\x72\x00\x00\x00\x00\x00')
    registry_value = fake.FakeWinRegistryValue(
        'MRU', data=data, data_type=definitions.REG_MULTI_SZ)

    expected_value_data = [
        '.bmp', '.contact', '.jnt', '.library-ms', '.lnk', '.rtf', '.txt',
        '.zip', 'Folder']

    value_data = registry_value.GetDataAsObject()
    self.assertEqual(value_data, expected_value_data)

    data = '\xed\x44'
    registry_value = fake.FakeWinRegistryValue(
        'MRU', data=data, data_type=definitions.REG_MULTI_SZ)

    with self.assertRaises(errors.WinRegistryValueError):
      registry_value.GetDataAsObject()

    registry_value = fake.FakeWinRegistryValue(
        'MRU', data=('bogus', 0), data_type=definitions.REG_SZ)

    with self.assertRaises(errors.WinRegistryValueError):
      registry_value.GetDataAsObject()

    registry_value = fake.FakeWinRegistryValue(
        'MRU', data=('bogus', 0), data_type=definitions.REG_MULTI_SZ)

    with self.assertRaises(errors.WinRegistryValueError):
      registry_value.GetDataAsObject()

  def testDataIsBinaryData(self):
    """Tests the DataIsBinaryData function."""
    registry_value = fake.FakeWinRegistryValue(
        'Count', data_type=definitions.REG_DWORD)

    self.assertFalse(registry_value.DataIsBinaryData())

    registry_value = fake.FakeWinRegistryValue(
        'MRUListEx', data_type=definitions.REG_BINARY)

    self.assertTrue(registry_value.DataIsBinaryData())

  def testDataIsInteger(self):
    """Tests the DataIsInteger function."""
    registry_value = fake.FakeWinRegistryValue(
        'MRUListEx', data_type=definitions.REG_BINARY)

    self.assertFalse(registry_value.DataIsInteger())

    registry_value = fake.FakeWinRegistryValue(
        'Count', data_type=definitions.REG_DWORD)

    self.assertTrue(registry_value.DataIsInteger())

  def testDataIsMultiString(self):
    """Tests the DataIsMultiString function."""
    registry_value = fake.FakeWinRegistryValue(
        'MRUListEx', data_type=definitions.REG_BINARY)

    self.assertFalse(registry_value.DataIsMultiString())

    registry_value = fake.FakeWinRegistryValue(
        'MRU', data_type=definitions.REG_MULTI_SZ)

    self.assertTrue(registry_value.DataIsMultiString())

  def testDataIsString(self):
    """Tests the DataIsString function."""
    registry_value = fake.FakeWinRegistryValue(
        'MRUListEx', data_type=definitions.REG_BINARY)

    self.assertFalse(registry_value.DataIsString())

    registry_value = fake.FakeWinRegistryValue(
        'MRU', data_type=definitions.REG_SZ)

    self.assertTrue(registry_value.DataIsString())


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Windows NT Registry (REGF) objects implementation using pyregf."""

from dfdatetime import filetime as dfdatetime_filetime
from dfdatetime import semantic_time as dfdatetime_semantic_time

import pyregf

from dfwinreg import definitions
from dfwinreg import errors
from dfwinreg import interface
from dfwinreg import key_paths


class REGFWinRegistryFile(interface.WinRegistryFile):
  """Implementation of a Windows Registry file using pyregf."""

  def __init__(
      self, ascii_codepage='cp1252', emulate_virtual_keys=True,
      key_path_prefix=''):
    """Initializes the Windows Registry file.

    Args:
      ascii_codepage (Optional[str]): ASCII string codepage.
      emulate_virtual_keys (Optional[bool]): True if virtual keys should be
          emulated.
      key_path_prefix (Optional[str]): Windows Registry key path prefix.
    """
    super(REGFWinRegistryFile, self).__init__(
        ascii_codepage=ascii_codepage, key_path_prefix=key_path_prefix)
    self._current_control_set_key = None
    self._current_control_set_key_path = None
    self._emulate_virtual_keys = emulate_virtual_keys
    self._file_object = None
    self._regf_file = pyregf.file()
    self._regf_file.set_ascii_codepage(ascii_codepage)

  def _GetCurrentControlSetKey(self):
    """Retrieves a current control set Windows Registry key.

    Returns:
      VirtualREGFWinRegistryKey: virtual current control set Windows
          Registry key or None if not available.
    """
    if not self._current_control_set_key:
      current_control_set_key = self._GetKeyByPathFromFile(
          self._current_control_set_key_path)
      if not current_control_set_key:
        return None

      name = 'CurrentControlSet'
      key_path = '\\'.join([self._key_path_prefix, name])
      self._current_control_set_key = VirtualREGFWinRegistryKey(
          name, current_control_set_key, key_path=key_path)

    return self._current_control_set_key

  def _GetCurrentControlSetKeyPath(self):
    """Retrieves the key path of the current control set key.

    Returns:
      str: key path of the current control set Windows Registry key or None
          if not available.
    """
    select_key = self._GetKeyByPathFromFile('\\Select')
    if not select_key:
      return None

    # To determine the current control set check:
    # 1. The "Current" value.
    # 2. The "Default" value.
    # 3. The "LastKnownGood" value.
    control_set = None
    for value_name in ('Current', 'Default', 'LastKnownGood'):
      value = select_key.get_value_by_name(value_name)
      if not value or value.type not in definitions.INTEGER_VALUE_TYPES:
        continue

      control_set = value.get_data_as_integer()
      # If the control set is 0 then we need to check the other values.
      if control_set > 0 or control_set <= 999:
        break

    if not control_set or control_set <= 0 or control_set > 999:
      return None

    return f'\\ControlSet{control_set:03d}'

  def _GetKeyByPathFromFile(self, key_path):
    """Retrieves the key for a specific path form the Windows Registry file.

    Args:
      key_path (str): Windows Registry key path relative to the file.

    Returns:
      pyregf.key: Registry key or None if not available.
    """
    try:
      return self._regf_file.get_key_by_path(key_path)
    except IOError:
      return None

  def Close(self):
    """Closes the Windows Registry file."""
    self._regf_file.close()
    self._file_object = None

  def GetKeyByPath(self, key_path):
    """Retrieves the key for a specific path.

    Args:
      key_path (str): Windows Registry key path.

    Returns:
      WinRegistryKey: Registry key or None if not available.
    """
    key_path_upper = key_path.upper()
    if key_path_upper.startswith(self._key_path_prefix_upper):
      relative_key_path = key_path[self._key_path_prefix_length:]
    elif key_path.startswith(definitions.KEY_PATH_SEPARATOR):
      relative_key_path = key_path
      key_path = ''.join([self._key_path_prefix, key_path])
    else:
      return None

    if relative_key_path and relative_key_path[0] == '\\':
      relative_key_path = relative_key_path[1:]

    relative_key_path_segments = relative_key_path.split('\\')

    if not relative_key_path:
      registry_key = self.GetRootKey()

    elif (self._emulate_virtual_keys and
          relative_key_path_segments[0].upper() == 'CURRENTCONTROLSET'):
      relative_key_path_segments.pop(0)

      registry_key = self._GetCurrentControlSetKey()
      if relative_key_path_segments:
        relative_sub_key_path = '\\'.join(relative_key_path_segments)
        registry_key = registry_key.GetSubkeyByPath(relative_sub_key_path)

    else:
      regf_key = self._GetKeyByPathFromFile(relative_key_path)
      if not regf_key:
        return None

      registry_key = REGFWinRegistryKey(regf_key, key_path=key_path)

    return registry_key

  def GetRootKey(self):
    """Retrieves the root key.

    Returns:
      WinRegistryKey: Windows Registry root key or None if not available.
    """
    regf_key = self._regf_file.get_root_key()
    if not regf_key:
      return None

    if self._current_control_set_key_path:
      current_control_set_key = self._GetKeyByPathFromFile(
          self._current_control_set_key_path)

      registry_key = VirtualREGFWinRegistryKey(
          '', regf_key, current_control_set_key=current_control_set_key,
          key_path=self._key_path_prefix)

    else:
      registry_key = REGFWinRegistryKey(
          regf_key, key_path=self._key_path_prefix)

    return registry_key

  def Open(self, file_object):
    """Opens the Windows Registry file using a file-like object.

    Args:
      file_object (file): file-like object.

    Returns:
      bool: True if successful or False if not.
    """
    self._file_object = file_object
    self._regf_file.open_file_object(self._file_object)

    if self._emulate_virtual_keys:
      self._current_control_set_key_path = self._GetCurrentControlSetKeyPath()

    return True


class REGFWinRegistryKey(interface.WinRegistryKey):
  """Implementation of a Windows Registry key using pyregf."""

  def __init__(self, pyregf_key, key_path=''):
    """Initializes a Windows Registry key.

    Args:
      pyregf_key (pyregf.key): pyregf key object.
      key_path (Optional[str]): Windows Registry key path.
    """
    super(REGFWinRegistryKey, self).__init__(key_path=key_path)
    self._pyregf_key = pyregf_key

  @property
  def class_name(self):
    """str: class name of the key or None if not available."""
    return self._pyregf_key.class_name

  @property
  def last_written_time(self):
    """dfdatetime.DateTimeValues: last written time."""
    timestamp = self._pyregf_key.get_last_written_time_as_integer()
    if timestamp == 0:
      return dfdatetime_semantic_time.SemanticTime('Not set')

    return dfdatetime_filetime.Filetime(timestamp=timestamp)

  @property
  def name(self):
    """str: name of the key."""
    return self._pyregf_key.name

  @property
  def number_of_subkeys(self):
    """int: number of subkeys within the key."""
    return self._pyregf_key.number_of_sub_keys

  @property
  def number_of_values(self):
    """int: number of values within the key."""
    return self._pyregf_key.number_of_values

  @property
  def offset(self):
    """int: offset of the key within the Windows Registry file or None."""
    return self._pyregf_key.offset

  def GetSubkeyByIndex(self, index):
    """Retrieves a subkey by index.

    Args:
      index (int): index of the subkey.

    Returns:
      WinRegistryKey: Windows Registry subkey.

    Raises:
      IndexError: if the index is out of bounds.
    """
    if index < 0 or index >= self._pyregf_key.number_of_sub_keys:
      raise IndexError('Index out of bounds.')

    pyregf_key = self._pyregf_key.get_sub_key(index)
    key_path = key_paths.JoinKeyPath([self._key_path, pyregf_key.name])
    return REGFWinRegistryKey(pyregf_key, key_path=key_path)

  def GetSubkeyByName(self, name):
    """Retrieves a subkey by name.

    Args:
      name (str): name of the subkey.

    Returns:
      WinRegistryKey: Windows Registry subkey or None if not found.
    """
    pyregf_key = self._pyregf_key.get_sub_key_by_name(name)
    if not pyregf_key:
      return None

    key_path = key_paths.JoinKeyPath([self._key_path, pyregf_key.name])
    return REGFWinRegistryKey(pyregf_key, key_path=key_path)

  def GetSubkeyByPath(self, key_path):
    """Retrieves a subkey by path.

    Args:
      key_path (str): path of the subkey.

    Returns:
      WinRegistryKey: Windows Registry subkey or None if not found.
    """
    pyregf_key = self._pyregf_key.get_sub_key_by_path(key_path)
    if not pyregf_key:
      return None

    key_path = key_paths.JoinKeyPath([self._key_path, key_path])
    return REGFWinRegistryKey(pyregf_key, key_path=key_path)

  def GetSubkeys(self):
    """Retrieves all subkeys within the key.

    Yields:
      WinRegistryKey: Windows Registry subkey.
    """
    for pyregf_key in self._pyregf_key.sub_keys:
      key_path = key_paths.JoinKeyPath([self._key_path, pyregf_key.name])
      yield REGFWinRegistryKey(pyregf_key, key_path=key_path)

  def GetValues(self):
    """Retrieves all values within the key.

    Yields:
      WinRegistryValue: Windows Registry value.
    """
    for pyregf_value in self._pyregf_key.values:
      yield REGFWinRegistryValue(pyregf_value)

  def GetValueByName(self, name):
    """Retrieves a value by name.

    Args:
      name (str): name of the value.

    Returns:
      WinRegistryValue: Windows Registry value or None if not found.
    """
    pyregf_value = self._pyregf_key.get_value_by_name(name)
    if not pyregf_value:
      return None

    return REGFWinRegistryValue(pyregf_value)


class VirtualREGFWinRegistryKey(REGFWinRegistryKey):
  """Implementation of a virtual Windows Registry key using pyregf.

  The virtual Windows Registry key are keys that do not exist on-disk but do
  exist at run-time, for example HKEY_LOCAL_MACHINE\\System\\CurrentControlSet.
  """

  def __init__(
      self, name, pyregf_key, current_control_set_key=None, key_path=''):
    """Initializes a virtual Windows Registry key.

    Args:
      name (str): name of the Windows Registry key.
      pyregf_key (pyregf.key): pyregf key object.
      current_control_set_key (Optional[pyregf.key]): pyregf key object of
          the control set key that represents CurrentControlSet.
      key_path (Optional[str]): Windows Registry key path.
    """
    super(VirtualREGFWinRegistryKey, self).__init__(
        pyregf_key, key_path=key_path)
    self._current_control_set_key = current_control_set_key
    self._name = name

  @property
  def name(self):
    """str: name of the key."""
    return self._name

  @property
  def number_of_subkeys(self):
    """int: number of subkeys within the key."""
    number_of_keys = self._pyregf_key.number_of_sub_keys
    if self._current_control_set_key:
      number_of_keys += 1
    return number_of_keys

  def GetSubkeyByIndex(self, index):
    """Retrieves a subkey by index.

    Args:
      index (int): index of the subkey.

    Returns:
      WinRegistryKey: Windows Registry subkey.

    Raises:
      IndexError: if the index is out of bounds.
    """
    if index < 0 or index >= self.number_of_subkeys:
      raise IndexError('Index out of bounds.')

    if (self._current_control_set_key and
        index == self._pyregf_key.number_of_sub_keys):
      name = 'CurrentControlSet'
      key_path = key_paths.JoinKeyPath([self._key_path, name])
      return VirtualREGFWinRegistryKey(
          name, self._current_control_set_key, key_path=key_path)

    pyregf_key = self._pyregf_key.get_sub_key(index)
    key_path = key_paths.JoinKeyPath([self._key_path, pyregf_key.name])
    return REGFWinRegistryKey(pyregf_key, key_path=key_path)

  def GetSubkeyByName(self, name):
    """Retrieves a subkey by name.

    Args:
      name (str): name of the subkey.

    Returns:
      WinRegistryKey: Windows Registry subkey or None if not found.
    """
    if self._current_control_set_key and name.upper() == 'CURRENTCONTROLSET':
      name = 'CurrentControlSet'
      key_path = key_paths.JoinKeyPath([self._key_path, name])
      return VirtualREGFWinRegistryKey(
          name, self._current_control_set_key, key_path=key_path)

    pyregf_key = self._pyregf_key.get_sub_key_by_name(name)
    if not pyregf_key:
      return None

    key_path = key_paths.JoinKeyPath([self._key_path, pyregf_key.name])
    return REGFWinRegistryKey(pyregf_key, key_path=key_path)

  def GetSubkeyByPath(self, key_path):
    """Retrieves a subkey by path.

    Args:
      key_path (str): path of the subkey.

    Returns:
      WinRegistryKey: Windows Registry subkey or None if not found.
    """
    if key_path and key_path[0] == '\\':
      key_path = key_path[1:]

    key_path_segments = key_path.split('\\')

    if (self._current_control_set_key and
        key_path_segments[0].upper() == 'CURRENTCONTROLSET'):
      key_path_segments.pop(0)

      if not key_path_segments:
        name = 'CurrentControlSet'
        key_path = key_paths.JoinKeyPath([self._key_path, name])
        return VirtualREGFWinRegistryKey(
            name, self._current_control_set_key, key_path=key_path)

      sub_key_path = '\\'.join(key_path_segments)
      pyregf_key = self._current_control_set_key.get_sub_key_by_path(
          sub_key_path)

    else:
      pyregf_key = self._pyregf_key.get_sub_key_by_path(key_path)

    if not pyregf_key:
      return None

    key_path = key_paths.JoinKeyPath([self._key_path, key_path])
    return REGFWinRegistryKey(pyregf_key, key_path=key_path)

  def GetSubkeys(self):
    """Retrieves all subkeys within the key.

    Yields:
      WinRegistryKey: Windows Registry subkey.
    """
    for pyregf_key in self._pyregf_key.sub_keys:
      key_path = key_paths.JoinKeyPath([self._key_path, pyregf_key.name])
      yield REGFWinRegistryKey(pyregf_key, key_path=key_path)

    if self._current_control_set_key:
      key_path = key_paths.JoinKeyPath([self._key_path, 'CurrentControlSet'])
      yield VirtualREGFWinRegistryKey(
          'CurrentControlSet', self._current_control_set_key,
          key_path=key_path)


class REGFWinRegistryValue(interface.WinRegistryValue):
  """Implementation of a Windows Registry value using pyregf."""

  # Note that missing-return-doc is broken for pylint 1.7.x
  # pylint: disable=missing-return-doc

  def __init__(self, pyregf_value):
    """Initializes a Windows Registry value.

    Args:
      pyregf_value (pyregf.value): pyregf value object.
    """
    super(REGFWinRegistryValue, self).__init__()
    self._pyregf_value = pyregf_value

  # Pylint 1.7.x seems to be get confused about properties.
  # pylint: disable=missing-return-type-doc
  @property
  def data(self):
    """bytes: value data as a byte string.

    Raises:
      WinRegistryValueError: if the value data cannot be read.
    """
    try:
      return self._pyregf_value.data
    except IOError as exception:
      raise errors.WinRegistryValueError((
          f'Unable to read data from value: {self._pyregf_value.name:s} '
          f'with error: {exception!s}'))

  @property
  def data_type(self):
    """int: data type."""
    return self._pyregf_value.type

  @property
  def name(self):
    """str: name of the value."""
    return self._pyregf_value.name

  @property
  def offset(self):
    """int: offset of the value within the Windows Registry file."""
    return self._pyregf_value.offset

  def GetDataAsObject(self):
    """Retrieves the data as an object.

    Returns:
      object: data as a Python type.

    Raises:
      WinRegistryValueError: if the value data cannot be read.
    """
    try:
      if self._pyregf_value.type in self._STRING_VALUE_TYPES:
        value_data = self._pyregf_value.get_data_as_string()

      elif self._pyregf_value.type in definitions.INTEGER_VALUE_TYPES:
        value_data = self._pyregf_value.get_data_as_integer()

      elif self._pyregf_value.type == definitions.REG_MULTI_SZ:
        value_data = self._pyregf_value.get_data_as_multi_string()

      else:
        value_data = self._pyregf_value.data

    except (IOError, OverflowError) as exception:
      raise errors.WinRegistryValueError((
          f'Unable to read data from value: {self._pyregf_value.name:s} '
          f'with error: {exception!s}'))

    return value_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the REGF Windows Registry back-end."""

import unittest

from dfwinreg import errors
from dfwinreg import regf

from tests import test_lib


class FakePyREGFKey(object):
  """Fake pyregf key for testing."""

  def __init__(self):
    """Initializes a fake pyregf key."""
    super(FakePyREGFKey, self).__init__()
    self.number_of_sub_keys = 1

  # pylint: disable=invalid-name,redundant-returns-doc,unused-argument

  def get_last_written_time_as_integer(self):
    """Retrieves the last written time as an integer.

    Returns:
      int: last written time, which will be 0 for testing.
    """
    return 0

  def get_sub_key(self, sub_key_index):
    """Retrieves a specific sub key.

    Returns:
      pyregf.key: sub key, which will be None for testing.
    """
    return None


class FakePyREGFValue(object):
  """Fake pyregf value for testing.

  Attributes:
    name (str): name of the value.
    type (str): value type.
  """

  def __init__(self, name='Test', value_type='REG_SZ'):
    """Initializes a fake pyregf value.

    Args:
      name (Optional[str]): name of the value.
      value_type (Optional[str]): value type.
    """
    super(FakePyREGFValue, self).__init__()
    self.name = name
    self.type = value_type

  # pylint: disable=missing-raises-doc

  @property
  def data(self):
    """bytes: value data."""
    raise IOError('raised for testing purposes.')


class REGFWinRegistryFileTest(test_lib.BaseTestCase):
  """Tests for the REGF Windows Registry file."""

  # pylint: disable=protected-access

  # TODO: add tests for _GetCurrentControlSetKey
  # TODO: add tests for _GetCurrentControlSetKeyPath
  # TODO: add tests for _GetKeyByPathFromFile

  def testOpenClose(self):
    """Tests the Open and Close functions."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)
      registry_file.Close()

  def testGetRootKey(self):
    """Tests the GetRootKey function."""
    # Test GetRootKey on NTUSER.DAT file
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetRootKey()
        self.assertIsNotNone(registry_key)
        self.assertIsInstance(registry_key, regf.REGFWinRegistryKey)
        self.assertEqual(registry_key.path, '\\')

      finally:
        registry_file.Close()

    # Test GetRootKey on SYSTEM file
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetRootKey()
        self.assertIsNotNone(registry_key)
        self.assertIsInstance(registry_key, regf.VirtualREGFWinRegistryKey)
        self.assertEqual(registry_key.path, '\\')

      finally:
        registry_file.Close()

    # Test GetRootKey on NTUSER.DAT.LOG file
    registry_file = regf.REGFWinRegistryFile()

    test_path = self._GetTestFilePath(['NTUSER.DAT.LOG'])
    self._SkipIfPathNotExists(test_path)

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        root_key = registry_file.GetRootKey()
        self.assertIsNone(root_key)

      finally:
        registry_file.Close()

  def testGetKeyByPath(self):
    """Tests the GetKeyByPath function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.name, '')
        self.assertEqual(registry_key.path, key_path)

        key_path = '\\ControlSet001'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.name, 'ControlSet001')
        self.assertEqual(registry_key.path, key_path)

        registry_key = registry_file.GetKeyByPath('ControlSet001')
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.name, 'ControlSet001')
        self.assertEqual(registry_key.path, key_path)

        key_path = '\\CurrentControlSet'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.name, 'CurrentControlSet')
        self.assertEqual(registry_key.path, key_path)

        key_path = '\\CurrentControlSet\\Enum'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.name, 'Enum')
        self.assertEqual(registry_key.path, key_path)

        key_path = '\\Bogus'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNone(registry_key)

      finally:
        registry_file.Close()

  def testRecurseKeys(self):
    """Tests the RecurseKeys function."""
    dat_test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(dat_test_path)

    log_test_path = self._GetTestFilePath(['NTUSER.DAT.LOG'])
    self._SkipIfPathNotExists(log_test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(dat_test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_keys = list(registry_file.RecurseKeys())

      finally:
        registry_file.Close()

    self.assertEqual(len(registry_keys), 1597)

    registry_file = regf.REGFWinRegistryFile()

    with open(log_test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_keys = list(registry_file.RecurseKeys())

      finally:
        registry_file.Close()

    self.assertEqual(len(registry_keys), 0)


class REGFWinRegistryKeyTest(test_lib.BaseTestCase):
  """Tests for the REGF Windows Registry key."""

  # pylint: disable=protected-access

  def testProperties(self):
    """Tests the properties functions."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\Software'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertIsNone(registry_key.class_name)
        self.assertEqual(registry_key.name, 'Software')
        self.assertEqual(registry_key.number_of_subkeys, 7)
        self.assertEqual(registry_key.number_of_values, 0)
        self.assertEqual(registry_key.offset, 4372)
        self.assertEqual(registry_key.path, key_path)

        self.assertIsNotNone(registry_key.last_written_time)
        timestamp = registry_key.last_written_time.timestamp
        self.assertEqual(timestamp, 131205170396534120)

        registry_key._pyregf_key = FakePyREGFKey()
        self.assertIsNotNone(registry_key.last_written_time)

        date_time_string = (
            registry_key.last_written_time.CopyToDateTimeString())
        self.assertEqual(date_time_string, 'Not set')

      finally:
        registry_file.Close()

  def testGetSubkeyByIndex(self):
    """Tests the GetSubkeyByIndex function."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile(
        key_path_prefix='HKEY_CURRENT_USER')

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetRootKey()

        key_name = 'AppEvents'
        sub_registry_key = registry_key.GetSubkeyByIndex(0)
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, key_name)

        expected_key_path = 'HKEY_CURRENT_USER\\AppEvents'
        self.assertEqual(sub_registry_key.path, expected_key_path)

        with self.assertRaises(IndexError):
          registry_key.GetSubkeyByIndex(-1)

      finally:
        registry_file.Close()

  def testGetSubkeyByName(self):
    """Tests the GetSubkeyByName function."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile(
        key_path_prefix='HKEY_CURRENT_USER')

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetRootKey()
        self.assertIsNotNone(registry_key)

        key_name = 'Software'
        sub_registry_key = registry_key.GetSubkeyByName(key_name)
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, key_name)

        expected_key_path = 'HKEY_CURRENT_USER\\Software'
        self.assertEqual(sub_registry_key.path, expected_key_path)

        key_name = 'Bogus'
        sub_registry_key = registry_key.GetSubkeyByName(key_name)
        self.assertIsNone(sub_registry_key)

      finally:
        registry_file.Close()

  def testGetSubkeyByPath(self):
    """Tests the GetSubkeyByPath function."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile(
        key_path_prefix='HKEY_CURRENT_USER')

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetRootKey()

        key_path = 'Software\\Microsoft'
        sub_registry_key = registry_key.GetSubkeyByPath(key_path)
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, 'Microsoft')

        expected_key_path = 'HKEY_CURRENT_USER\\Software\\Microsoft'
        self.assertEqual(sub_registry_key.path, expected_key_path)

        key_path = 'Software\\Bogus'
        sub_registry_key = registry_key.GetSubkeyByPath(key_path)
        self.assertIsNone(sub_registry_key)

      finally:
        registry_file.Close()

  def testGetSubkeys(self):
    """Tests the GetSubkeys function."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\Software'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)

        sub_registry_keys = list(registry_key.GetSubkeys())
        self.assertEqual(len(sub_registry_keys), 7)

      finally:
        registry_file.Close()

  def testGetValueByName(self):
    """Tests the GetValueByName function."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetKeyByPath('\\Console')
        self.assertIsNotNone(registry_key)

        value_name = 'ColorTable14'
        registry_value = registry_key.GetValueByName(value_name)
        self.assertIsNotNone(registry_value)
        self.assertEqual(registry_value.name, value_name)

        value_name = 'Bogus'
        registry_value = registry_key.GetValueByName(value_name)
        self.assertIsNone(registry_value)

        # Test retrieving the default (or nameless) value.
        registry_key = registry_file.GetKeyByPath(
            '\\AppEvents\\EventLabels\\.Default')
        self.assertIsNotNone(registry_key)

        registry_value = registry_key.GetValueByName('')
        self.assertIsNotNone(registry_value)
        self.assertIsNone(registry_value.name)

      finally:
        registry_file.Close()

  def testGetValues(self):
    """Tests the GetValues function."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\Console'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)

        values = list(registry_key.GetValues())
        self.assertEqual(len(values), 37)

      finally:
        registry_file.Close()

  def testRecurseKeys(self):
    """Tests the RecurseKeys function."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\Software'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)

        registry_keys = list(registry_key.RecurseKeys())

      finally:
        registry_file.Close()

    self.assertEqual(len(registry_keys), 1219)


class VirtualREGFWinRegistryKeyTest(test_lib.BaseTestCase):
  """Tests for the virtual REGF Windows Registry key."""

  # pylint: disable=protected-access

  def testGetSubkeyByIndex(self):
    """Tests the GetSubkeyByIndex function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.path, key_path)
        self.assertIsInstance(registry_key, regf.VirtualREGFWinRegistryKey)

        self.assertEqual(registry_key.number_of_subkeys, 9)

        sub_registry_key = registry_key.GetSubkeyByIndex(0)
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, 'ControlSet001')
        self.assertIsInstance(sub_registry_key, regf.REGFWinRegistryKey)

        sub_registry_key = registry_key.GetSubkeyByIndex(8)
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, 'CurrentControlSet')
        self.assertIsInstance(sub_registry_key, regf.VirtualREGFWinRegistryKey)

        self.assertEqual(sub_registry_key.number_of_subkeys, 5)

        with self.assertRaises(IndexError):
          registry_key.GetSubkeyByIndex(9)

      finally:
        registry_file.Close()

  def testGetSubkeyByName(self):
    """Tests the GetSubkeyByName function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.path, key_path)
        self.assertIsInstance(registry_key, regf.VirtualREGFWinRegistryKey)

        self.assertEqual(registry_key.number_of_subkeys, 9)

        sub_registry_key = registry_key.GetSubkeyByName('ControlSet001')
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, 'ControlSet001')
        self.assertIsInstance(sub_registry_key, regf.REGFWinRegistryKey)

        sub_registry_key = registry_key.GetSubkeyByName('CurrentControlSet')
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, 'CurrentControlSet')
        self.assertIsInstance(sub_registry_key, regf.VirtualREGFWinRegistryKey)

        self.assertEqual(sub_registry_key.number_of_subkeys, 5)

        # Ensure the virtual CurrentControlSet key does not return another copy
        # of itself.
        test_registry_key = sub_registry_key.GetSubkeyByName(
            'CurrentControlSet')
        self.assertIsNone(test_registry_key)

      finally:
        registry_file.Close()

  def testGetSubkeyByPath(self):
    """Tests the GetSubkeyByPath function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.path, key_path)
        self.assertIsInstance(registry_key, regf.VirtualREGFWinRegistryKey)

        self.assertEqual(registry_key.number_of_subkeys, 9)

        sub_registry_key = registry_key.GetSubkeyByPath('\\ControlSet001')
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, 'ControlSet001')
        self.assertIsInstance(sub_registry_key, regf.REGFWinRegistryKey)

        sub_registry_key = registry_key.GetSubkeyByPath(
            '\\ControlSet001\\Enum')
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, 'Enum')
        self.assertEqual(sub_registry_key.path, '\\ControlSet001\\Enum')
        self.assertIsInstance(sub_registry_key, regf.REGFWinRegistryKey)

        sub_registry_key = registry_key.GetSubkeyByPath('\\CurrentControlSet')
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, 'CurrentControlSet')
        self.assertIsInstance(sub_registry_key, regf.VirtualREGFWinRegistryKey)

        sub_registry_key = registry_key.GetSubkeyByPath(
            '\\CurrentControlSet\\Enum')
        self.assertIsNotNone(sub_registry_key)
        self.assertEqual(sub_registry_key.name, 'Enum')
        self.assertEqual(sub_registry_key.path, '\\CurrentControlSet\\Enum')
        self.assertIsInstance(sub_registry_key, regf.REGFWinRegistryKey)

      finally:
        registry_file.Close()

  def testGetSubkeys(self):
    """Tests the GetSubkeys function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        key_path = '\\'
        registry_key = registry_file.GetKeyByPath(key_path)
        self.assertIsNotNone(registry_key)
        self.assertEqual(registry_key.path, key_path)
        self.assertIsInstance(registry_key, regf.VirtualREGFWinRegistryKey)

        self.assertEqual(registry_key.number_of_subkeys, 9)

        expected_subkey_names = [
            'ControlSet001',
            'DriverDatabase',
            'HardwareConfig',
            'MountedDevices',
            'RNG',
            'Select',
            'Setup',
            'WPA',
            'CurrentControlSet']

        subkey_names = [subkey.name for subkey in registry_key.GetSubkeys()]
        self.assertEqual(subkey_names, expected_subkey_names)

        sub_registry_key = registry_key.GetSubkeyByPath('\\CurrentControlSet')
        self.assertIsNotNone(sub_registry_key)

        self.assertEqual(sub_registry_key.number_of_subkeys, 5)

        expected_subkey_names = [
            'Control',
            'Enum',
            'Hardware Profiles',
            'Policies',
            'Services']

        subkey_names = [subkey.name for subkey in sub_registry_key.GetSubkeys()]
        self.assertEqual(subkey_names, expected_subkey_names)

      finally:
        registry_file.Close()


class REGFWinRegistryValueTest(test_lib.BaseTestCase):
  """Tests for the REGF Windows Registry value."""

  # pylint: disable=protected-access

  def testProperties(self):
    """Tests the properties functions on a NTUSER.DAT file."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetKeyByPath('\\Console')
        self.assertIsNotNone(registry_key)

        value_name = 'ColorTable14'
        registry_value = registry_key.GetValueByName(value_name)
        expected_data = b'\xff\xff\x00\x00'

        self.assertIsNotNone(registry_value)
        self.assertEqual(registry_value.data_type, 4)
        self.assertEqual(registry_value.data_type_string, 'REG_DWORD_LE')
        self.assertEqual(registry_value.GetDataAsObject(), 65535)
        self.assertEqual(registry_value.name, value_name)
        self.assertEqual(registry_value.offset, 105212)
        self.assertEqual(registry_value.data, expected_data)

        registry_key = registry_file.GetKeyByPath(
            '\\AppEvents\\EventLabels\\CriticalBatteryAlarm')
        self.assertIsNotNone(registry_key)

        value_name = 'DispFileName'
        registry_value = registry_key.GetValueByName(value_name)
        expected_data = (
            b'@\x00m\x00m\x00r\x00e\x00s\x00.\x00d\x00l\x00l\x00,\x00-\x005'
            b'\x008\x002\x007\x00\x00\x00')

        self.assertIsNotNone(registry_value)
        self.assertEqual(registry_value.data_type, 1)
        self.assertEqual(registry_value.data_type_string, 'REG_SZ')
        self.assertEqual(registry_value.GetDataAsObject(), '@mmres.dll,-5827')
        self.assertEqual(registry_value.name, value_name)
        self.assertEqual(registry_value.offset, 62028)
        self.assertEqual(registry_value.data, expected_data)

        registry_key = registry_file.GetKeyByPath('\\Control Panel\\Appearance')
        self.assertIsNotNone(registry_key)

        value_name = 'SchemeLangID'
        registry_value = registry_key.GetValueByName(value_name)
        expected_data = b'\x00\x00'

        self.assertIsNotNone(registry_value)
        self.assertEqual(registry_value.data_type, 3)
        self.assertEqual(registry_value.data_type_string, 'REG_BINARY')
        self.assertEqual(registry_value.GetDataAsObject(), expected_data)
        self.assertEqual(registry_value.name, value_name)
        self.assertEqual(registry_value.offset, 46468)
        self.assertEqual(registry_value.data, expected_data)

        registry_value._pyregf_value = FakePyREGFValue()
        with self.assertRaises(errors.WinRegistryValueError):
          _ = registry_value.data

      finally:
        registry_file.Close()

  def testGetDataAsObject(self):
    """Tests the GetDataAsObject function."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    registry_file = regf.REGFWinRegistryFile()

    with open(test_path, 'rb') as file_object:
      registry_file.Open(file_object)

      try:
        registry_key = registry_file.GetKeyByPath('\\Console')
        self.assertIsNotNone(registry_key)

        registry_value = registry_key.GetValueByName('ColorTable14')
        self.assertIsNotNone(registry_value)

        data_object = registry_value.GetDataAsObject()
        self.assertEqual(data_object, 65535)

        registry_value._pyregf_value = FakePyREGFValue(value_type='REG_SZ')
        with self.assertRaises(errors.WinRegistryValueError):
          registry_value.GetDataAsObject()

        registry_value._pyregf_value = FakePyREGFValue(
            value_type='REG_DWORD_LE')
        with self.assertRaises(errors.WinRegistryValueError):
          registry_value.GetDataAsObject()

        registry_value._pyregf_value = FakePyREGFValue(
            value_type='REG_MULTI_SZ')
        with self.assertRaises(errors.WinRegistryValueError):
          registry_value.GetDataAsObject()

        # Test REG_MULTI_SZ without additional empty string.
        registry_key = registry_file.GetKeyByPath(
            '\\Control Panel\\International\\User Profile')
        self.assertIsNotNone(registry_key)

        registry_value = registry_key.GetValueByName('Languages')
        self.assertIsNotNone(registry_value)

        data_object = registry_value.GetDataAsObject()
        self.assertEqual(len(data_object), 1)

        # Test REG_MULTI_SZ with additional empty string.
        registry_key = registry_file.GetKeyByPath(
            '\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\'
            'Discardable\\PostSetup\\ShellNew')
        self.assertIsNotNone(registry_key)

        registry_value = registry_key.GetValueByName('Classes')
        self.assertIsNotNone(registry_value)

        data_object = registry_value.GetDataAsObject()
        self.assertEqual(len(data_object), 9)

      finally:
        registry_file.Close()


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the Windows Registry searcher."""

import unittest

from dfwinreg import fake
from dfwinreg import registry
from dfwinreg import registry_searcher

from tests import registry as test_registry
from tests import test_lib


class FindSpecTest(test_lib.BaseTestCase):
  """Tests for the find specification."""

  # pylint: disable=protected-access

  def testInitialize(self):
    """Tests the __init__ function."""
    find_spec = registry_searcher.FindSpec()
    self.assertIsNotNone(find_spec)

    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Microsoft')
    self.assertIsNotNone(find_spec)

    find_spec = registry_searcher.FindSpec(
        key_path=['HKEY_CURRENT_USER', 'Software', 'Microsoft'])
    self.assertIsNotNone(find_spec)

    find_spec = registry_searcher.FindSpec(
        key_path_glob='HKEY_CURRENT_USER\\*\\Microsoft')
    self.assertIsNotNone(find_spec)

    find_spec = registry_searcher.FindSpec(
        key_path_glob=['HKEY_CURRENT_USER', '*', 'Microsoft'])
    self.assertIsNotNone(find_spec)

    find_spec = registry_searcher.FindSpec(
        key_path_regex='HKEY_CURRENT_USER\\.*\\Microsoft')
    self.assertIsNotNone(find_spec)

    find_spec = registry_searcher.FindSpec(
        key_path_regex=['HKEY_CURRENT_USER', '.*', 'Microsoft'])
    self.assertIsNotNone(find_spec)

    with self.assertRaises(TypeError):
      registry_searcher.FindSpec(key_path=('bogus', 0))

    with self.assertRaises(TypeError):
      registry_searcher.FindSpec(key_path_glob=('bogus', 0))

    with self.assertRaises(TypeError):
      registry_searcher.FindSpec(key_path_regex=('bogus', 0))

    with self.assertRaises(ValueError):
      registry_searcher.FindSpec(
          key_path='HKEY_CURRENT_USER\\Software\\Microsoft',
          key_path_glob='HKEY_CURRENT_USER\\*\\Microsoft')

  def testCheckKeyPath(self):
    """Tests the _CheckKeyPath function."""
    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    registry_key = fake.FakeWinRegistryKey(
        'Microsoft', key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec._CheckKeyPath(registry_key, 3)
    self.assertTrue(result)

    result = find_spec._CheckKeyPath(registry_key, 0)
    self.assertTrue(result)

    # Test incorrect search depth.
    result = find_spec._CheckKeyPath(registry_key, 1)
    self.assertFalse(result)

    # Test invalid search depth.
    result = find_spec._CheckKeyPath(registry_key, -1)
    self.assertFalse(result)

    result = find_spec._CheckKeyPath(registry_key, 99)
    self.assertFalse(result)

    # Test find specification with regular expression.
    find_spec = registry_searcher.FindSpec(
        key_path_regex=['HKEY_CURRENT_USER', 'Software', 'Microsoft'])

    registry_key = fake.FakeWinRegistryKey(
        'Microsoft', key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec._CheckKeyPath(registry_key, 3)
    self.assertTrue(result)

    # Test find specification without key_path_segments.
    find_spec._key_path_segments = None
    result = find_spec._CheckKeyPath(registry_key, 3)
    self.assertFalse(result)

    # Test find specification with invalid regular expression.
    find_spec = registry_searcher.FindSpec(
        key_path_regex=['HKEY_CURRENT_USER', 'Software', 'Mi(rosoft'])

    registry_key = fake.FakeWinRegistryKey(
        'Microsoft', key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec._CheckKeyPath(registry_key, 3)
    self.assertFalse(result)

  def testCompareWithKeyPathSegment(self):
    """Test the _CompareWithKeyPathSegment function."""
    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec._CompareWithKeyPathSegment('Microsoft', 2)
    self.assertTrue(result)

    result = find_spec._CompareWithKeyPathSegment('Microsoft', 1)
    self.assertFalse(result)

    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Bogus')

    result = find_spec._CompareWithKeyPathSegment('Microsoft', 2)
    self.assertFalse(result)

  def testAtLastKeyPathSegment(self):
    """Test the AtLastKeyPathSegment function."""
    find_spec = registry_searcher.FindSpec()

    result = find_spec.AtLastKeyPathSegment(2)
    self.assertFalse(result)

    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec.AtLastKeyPathSegment(0)
    self.assertFalse(result)

    result = find_spec.AtLastKeyPathSegment(2)
    self.assertTrue(result)

    result = find_spec.AtLastKeyPathSegment(5)
    self.assertTrue(result)

  def testAtMaximumDepth(self):
    """Tests the AtMaximumDepth function."""
    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec.AtMaximumDepth(1)
    self.assertFalse(result)

    result = find_spec.AtMaximumDepth(5)
    self.assertTrue(result)

  def testCompareKeyPath(self):
    """Test the CompareKeyPath function."""
    registry_key = fake.FakeWinRegistryKey(
        'Microsoft', key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec.CompareKeyPath(registry_key)
    self.assertTrue(result)

    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Bogus')

    result = find_spec.CompareKeyPath(registry_key)
    self.assertFalse(result)

  def testCompareNameWithKeyPathSegment(self):
    """Test the CompareNameWithKeyPathSegment function."""
    registry_key = fake.FakeWinRegistryKey(
        'Microsoft', key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec.CompareNameWithKeyPathSegment(registry_key, 2)
    self.assertTrue(result)

    result = find_spec.CompareNameWithKeyPathSegment(registry_key, 1)
    self.assertFalse(result)

    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Bogus')

    result = find_spec.CompareNameWithKeyPathSegment(registry_key, 2)
    self.assertFalse(result)

  def testHasKeyPath(self):
    """Test the HasKeyPath function."""
    find_spec = registry_searcher.FindSpec()

    result = find_spec.HasKeyPath()
    self.assertFalse(result)

    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec.HasKeyPath()
    self.assertTrue(result)

  def testIsLastKeyPathSegment(self):
    """Test the IsLastKeyPathSegment function."""
    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec.IsLastKeyPathSegment(0)
    self.assertFalse(result)

    result = find_spec.IsLastKeyPathSegment(2)
    self.assertTrue(result)

    result = find_spec.IsLastKeyPathSegment(5)
    self.assertFalse(result)

  def testMatches(self):
    """Tests the Matches function."""
    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    registry_key = fake.FakeWinRegistryKey(
        'Microsoft', key_path='HKEY_CURRENT_USER\\Software\\Microsoft')

    result = find_spec.Matches(registry_key, 3)
    self.assertEqual(result, (True, True))

    result = find_spec.Matches(registry_key, 1)
    self.assertEqual(result, (False, False))

    result = find_spec.Matches(registry_key, 0)
    self.assertEqual(result, (False, True))

    # Test find specification without key_path_segments.
    find_spec._key_path_segments = None
    result = find_spec.Matches(registry_key, 3)
    self.assertEqual(result, (True, None))


class WinRegistrySearcherTest(test_lib.BaseTestCase):
  """Tests for the Windows Registry searcher."""

  # pylint: disable=protected-access

  def testInitialize(self):
    """Tests the __init__ function."""
    with self.assertRaises(ValueError):
      registry_searcher.WinRegistrySearcher(None)

  # TODO: add tests for _FindInKey

  def testFind(self):
    """Tests the Find function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    win_registry = registry.WinRegistry(
        registry_file_reader=test_registry.TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)

    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    searcher = registry_searcher.WinRegistrySearcher(win_registry)

    # Test with key path.
    find_spec = registry_searcher.FindSpec(
        key_path='HKEY_LOCAL_MACHINE\\System\\ControlSet001')

    expected_key_paths = ['HKEY_LOCAL_MACHINE\\System\\ControlSet001']
    key_paths = list(searcher.Find(find_specs=[find_spec]))
    self.assertEqual(key_paths, expected_key_paths)

    # Test with key path glob.
    find_spec = registry_searcher.FindSpec(
        key_path_glob='HKEY_LOCAL_MACHINE\\System\\ControlSet001\\*')

    expected_key_paths = [
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control',
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Enum',
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Hardware Profiles',
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Policies',
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Services']
    key_paths = list(searcher.Find(find_specs=[find_spec]))
    self.assertEqual(key_paths, expected_key_paths)

    # Test with key path regular expression.
    find_spec = registry_searcher.FindSpec(
        key_path_regex=[
            'HKEY_LOCAL_MACHINE', 'System', 'ControlSet001', '.*'])

    expected_key_paths = [
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control',
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Enum',
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Hardware Profiles',
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Policies',
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Services']
    key_paths = list(searcher.Find(find_specs=[find_spec]))
    self.assertEqual(key_paths, expected_key_paths)

    # Test with CurrentControlSet.
    find_spec = registry_searcher.FindSpec(
        key_path_glob='HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\*')

    expected_key_paths = [
        'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control',
        'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Enum',
        'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Hardware Profiles',
        'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Policies',
        'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Services']
    key_paths = list(searcher.Find(find_specs=[find_spec]))
    self.assertEqual(key_paths, expected_key_paths)

    # Test without find specifications.
    win_registry = registry.WinRegistry(
        registry_file_reader=test_registry.TestREGFWinRegistryFileReader(
            emulate_virtual_keys=False))

    registry_file = win_registry._OpenFile(test_path)

    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    searcher = registry_searcher.WinRegistrySearcher(win_registry)
    key_paths = list(searcher.Find())
    self.assertEqual(len(key_paths), 21512)

  def testGetKeyByPath(self):
    """Tests the GetKeyByPath function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    win_registry = registry.WinRegistry(
        registry_file_reader=test_registry.TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)

    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    searcher = registry_searcher.WinRegistrySearcher(win_registry)

    registry_key = searcher.GetKeyByPath(
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control')
    self.assertIsNotNone(registry_key)

    registry_key = searcher.GetKeyByPath(
        'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control')
    self.assertIsNotNone(registry_key)

  def testSplitKeyPath(self):
    """Tests the SplitKeyPath function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    win_registry = registry.WinRegistry(
        registry_file_reader=test_registry.TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)

    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    searcher = registry_searcher.WinRegistrySearcher(win_registry)

    path_segments = searcher.SplitKeyPath(
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control')
    self.assertEqual(len(path_segments), 4)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Functions and classes for testing."""

import os
import unittest


# The path to top of the dfWinReg source tree.
PROJECT_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))

# The paths below are all derived from the project path directory.
# They are enumerated explicitly here so that they can be overwritten for
# compatibility with different build systems.
TEST_DATA_PATH = os.path.join(PROJECT_PATH, 'test_data')


class BaseTestCase(unittest.TestCase):
  """The base test case."""

  # Show full diff results.
  maxDiff = None

  def _GetTestFilePath(self, path_segments):
    """Retrieves the path of a test file relative to the test data directory.

    Args:
      path_segments (list[str]): path segments inside the test data directory.

    Returns:
      str: path of the test file.
    """
    # Note that we need to pass the individual path segments to os.path.join
    # and not a list.
    return os.path.join(TEST_DATA_PATH, *path_segments)

  def _SkipIfPathNotExists(self, path):
    """Skips the test if the path does not exist.

    Args:
      path (str): path of a test file.

    Raises:
      SkipTest: if the path does not exist and the test should be skipped.
    """
    if not os.path.exists(path):
      filename = os.path.basename(path)
      raise unittest.SkipTest(f'missing test file: {filename:s}')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""This file contains the tests for the Windows Registry library."""

import os
import unittest

from dfwinreg import definitions
from dfwinreg import fake
from dfwinreg import interface
from dfwinreg import regf
from dfwinreg import registry

from tests import test_lib


class TestWinRegistry(registry.WinRegistry):
  """Windows Registry for testing."""

  def GetKeyByPath(self, key_path):
    """Retrieves the key for a specific path.

    Args:
      key_path (str): Windows Registry key path.

    Returns:
      WinRegistryKey: Windows Registry key or None if not available.

    Raises:
      RuntimeError: if the root key is not supported.
    """
    if key_path == 'HKEY_LOCAL_MACHINE\\System\\Select':
      registry_key = fake.FakeWinRegistryKey(
          'Select', key_path='HKEY_LOCAL_MACHINE\\System',
          last_written_time=0)

      registry_value = fake.FakeWinRegistryValue(
          'Current', data=b'DATA', data_type=definitions.REG_BINARY)
      registry_key.AddValue(registry_value)

      registry_value = fake.FakeWinRegistryValue(
          'Default', data=b'\xff\xff\xff\xff', data_type=definitions.REG_DWORD)
      registry_key.AddValue(registry_value)

      registry_value = fake.FakeWinRegistryValue(
          'LastKnownGood', data=b'\x01\x00\x00\x00',
          data_type=definitions.REG_DWORD)
      registry_key.AddValue(registry_value)

      return registry_key

    return super(TestWinRegistry, self).GetKeyByPath(key_path)


class TestWinRegistryKeyPathPrefixMismatch(registry.WinRegistry):
  """Windows Registry for testing key path prefix mismatch."""

  def _GetFileByPath(self, key_path_upper):
    """Retrieves a Windows Registry file for a specific path.

    Args:
      key_path_upper (str): Windows Registry key path, in upper case with
          a resolved root key alias.

    Returns:
      tuple: consists:

        str: upper case key path prefix
        WinRegistryFile: corresponding Windows Registry file or None if not
            available.
    """
    _, registry_file = super(
        TestWinRegistryKeyPathPrefixMismatch, self)._GetFileByPath(
            key_path_upper)
    return 'BOGUS', registry_file


class TestREGFWinRegistryFileReader(interface.WinRegistryFileReader):
  """Single file Windows Registry file reader."""

  def __init__(self, emulate_virtual_keys=True):
    """Initializes a file Windows Registry file reader.

    Args:
      emulate_virtual_keys (Optional[bool]): True if virtual keys should be
          emulated.
    """
    super(TestREGFWinRegistryFileReader, self).__init__()
    self._emulate_virtual_keys = emulate_virtual_keys

  def Open(self, path, ascii_codepage='cp1252'):
    """Opens the Windows Registry file specified by the path.

    Args:
      path (str): path of the Windows Registry file.
      ascii_codepage (Optional[str]): ASCII string codepage.

    Returns:
      WinRegistryFile: Windows Registry file or None.
    """
    if not os.path.exists(path):
      return None

    registry_file = regf.REGFWinRegistryFile(
        ascii_codepage=ascii_codepage,
        emulate_virtual_keys=self._emulate_virtual_keys)

    file_object = open(path, 'rb')  # pylint: disable=consider-using-with

    try:
      # If open is successful Registry file will manage the file object.
      registry_file.Open(file_object)
    except IOError:
      file_object.close()
      registry_file = None

    return registry_file


class TestREGFWinRegistryFileReaderMapped(TestREGFWinRegistryFileReader):
  """Single file Windows Registry file reader that maps Windows paths."""

  def Open(self, path, ascii_codepage='cp1252'):
    """Opens the Windows Registry file specified by the path.

    Args:
      path (str): path of the Windows Registry file.
      ascii_codepage (Optional[str]): ASCII string codepage.

    Returns:
      WinRegistryFile: Windows Registry file or None.

    Raises:
      SkipTest: if the Windows Registry file does not exist and the test
          should be skipped.
    """
    if path == '%SystemRoot%\\System32\\config\\SYSTEM':
      path = os.path.join(test_lib.TEST_DATA_PATH, 'SYSTEM')
    elif path == '%UserProfile%\\NTUSER.DAT':
      path = os.path.join(test_lib.TEST_DATA_PATH, 'NTUSER.DAT')

    return super(TestREGFWinRegistryFileReaderMapped, self).Open(
        path, ascii_codepage=ascii_codepage)


class RegistryTest(test_lib.BaseTestCase):
  """Tests for the Windows Registry library."""

  # pylint: disable=protected-access

  def testGetCachedFileByPath(self):
    """Tests the _GetCachedFileByPath function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    win_registry = registry.WinRegistry()

    # Note that _GetCachedFileByPath expects the key path to be in
    # upper case.
    key_path = 'HKEY_LOCAL_MACHINE\\SYSTEM'
    key_path_prefix, registry_file = win_registry._GetCachedFileByPath(
        key_path)
    self.assertIsNone(key_path_prefix)
    self.assertIsNone(registry_file)

    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)
    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    key_path_prefix, registry_file = win_registry._GetCachedFileByPath(
        key_path)
    self.assertEqual(key_path_prefix, key_path)
    self.assertIsNotNone(registry_file)

  # TODO: add tests for _GetCachedUserFileByPath

  def testGetCandidateFileMappingsByPath(self):
    """Tests the _GetCandidateFileMappingsByPath function."""
    win_registry = registry.WinRegistry()

    # Note that _GetCandidateFileMappingsByPath expects the key path to be in
    # upper case.
    key_path = 'HKEY_USERS'
    mappings = list(win_registry._GetCandidateFileMappingsByPath(key_path))
    self.assertEqual(len(mappings), 1)

    key_path = 'HKEY_BOGUS\\SYSTEM'
    mappings = list(win_registry._GetCandidateFileMappingsByPath(key_path))
    self.assertEqual(len(mappings), 0)

    key_path = 'HKEY_CURRENT_USER\\SOFTWARE\\CLASSES'
    mappings = list(win_registry._GetCandidateFileMappingsByPath(key_path))
    self.assertEqual(len(mappings), 3)

  def testGetKeyByPathFromFile(self):
    """Tests the _GetKeyByPathFromFile function."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)

    win_registry = registry.WinRegistry()
    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    # Test an existing key.
    registry_key = win_registry._GetKeyByPathFromFile(
        'HKEY_CURRENT_USER\\Software\\Microsoft')
    self.assertIsNotNone(registry_key)

    # Test a non-existing key.
    registry_key = win_registry._GetKeyByPathFromFile(
        'HKEY_CURRENT_USER\\Software\\Bogus')
    self.assertIsNone(registry_key)

    # Test a non-existing key outside the Registry file.
    registry_key = win_registry._GetKeyByPathFromFile(
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001')
    self.assertIsNone(registry_key)

  def testGetUsersVirtualKey(self):
    """Tests the _GetUsersVirtualKey function."""
    ntuser_test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(ntuser_test_path)

    software_test_path = self._GetTestFilePath(['SOFTWARE'])
    self._SkipIfPathNotExists(software_test_path)

    win_registry = registry.WinRegistry()

    registry_key = win_registry._GetUsersVirtualKey('\\S-1-5-18')
    self.assertIsNone(registry_key)

    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(ntuser_test_path)
    profile_path = '%SystemRoot%\\System32\\config\\systemprofile\\NTUSER.DAT'
    win_registry.MapUserFile(profile_path, registry_file)

    registry_file = win_registry._OpenFile(software_test_path)
    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    registry_key = win_registry._GetUsersVirtualKey('\\S-1-5-18')
    self.assertIsNotNone(registry_key)

    expected_key_path = 'HKEY_USERS\\S-1-5-18'
    self.assertEqual(registry_key.path, expected_key_path)

    registry_key = win_registry._GetUsersVirtualKey(
        '\\S-1-5-18\\Software\\Microsoft\\Windows\\CurrentVersion')
    self.assertIsNotNone(registry_key)

    expected_key_path = (
        'HKEY_USERS\\S-1-5-18\\Software\\Microsoft\\Windows\\CurrentVersion')
    self.assertEqual(registry_key.path, expected_key_path)

    registry_key = win_registry._GetUsersVirtualKey('\\.DEFAULT')
    self.assertIsNotNone(registry_key)

    expected_key_path = 'HKEY_USERS\\.DEFAULT'
    self.assertEqual(registry_key.path, expected_key_path)

    with self.assertRaises(RuntimeError):
      win_registry._GetUsersVirtualKey('.DEFAULT')

  def testGetFileByPath(self):
    """Tests the _GetFileByPath function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    key_path = 'HKEY_LOCAL_MACHINE\\SYSTEM'

    # Test mapped file with key path prefix.
    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)
    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    key_path_prefix, registry_file = win_registry._GetFileByPath(key_path)
    self.assertEqual(key_path_prefix, key_path)
    self.assertIsNotNone(registry_file)

    # Test mapped file without key path prefix.
    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)
    win_registry.MapFile('', registry_file)

    key_path_prefix, registry_file = win_registry._GetFileByPath(key_path)
    self.assertEqual(key_path_prefix, key_path)
    self.assertIsNone(registry_file)

    # Test without mapped file.
    win_registry = registry.WinRegistry()

    key_path_prefix, registry_file = win_registry._GetFileByPath(key_path)
    self.assertEqual(key_path_prefix, key_path)
    self.assertIsNone(registry_file)

    # Tests open file based on predefined mapping.
    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReaderMapped())

    key_path_prefix, registry_file = win_registry._GetFileByPath(key_path)
    self.assertEqual(key_path_prefix, key_path)
    self.assertIsNotNone(registry_file)

  def testGetKeyByPathOnNTUserDat(self):
    """Tests the GetKeyByPath function on a NTUSER.DAT file."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)

    win_registry = registry.WinRegistry()
    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

    # Test an existing key.
    registry_key = win_registry.GetKeyByPath(
        'HKEY_CURRENT_USER\\Software\\Microsoft')
    self.assertIsNotNone(registry_key)

    # Test a non-existing key.
    registry_key = win_registry.GetKeyByPath(
        'HKEY_CURRENT_USER\\Software\\Bogus')
    self.assertIsNone(registry_key)

    # Test a non-existing root.
    with self.assertRaises(RuntimeError):
      win_registry.GetKeyByPath('HKEY_BOGUS\\Software\\Bogus')

    # Test a non-existing key outside the Registry file.
    registry_key = win_registry.GetKeyByPath(
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001')
    self.assertIsNone(registry_key)

  def testGetKeyByPathOnSystem(self):
    """Tests the GetKeyByPath function on a SYSTEM file."""
    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReaderMapped())

    # Test an existing key.
    registry_key = win_registry.GetKeyByPath(
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001')
    self.assertIsNotNone(registry_key)

    self.assertEqual(
        registry_key.path, 'HKEY_LOCAL_MACHINE\\SYSTEM\\ControlSet001')

    registry_key = win_registry.GetKeyByPath(
        'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Enum')
    self.assertIsNotNone(registry_key)

    self.assertEqual(
        registry_key.path, 'HKEY_LOCAL_MACHINE\\SYSTEM\\ControlSet001\\Enum')

    # Test a virtual key.
    registry_key = win_registry.GetKeyByPath(
        'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet')
    self.assertIsNotNone(registry_key)

    self.assertEqual(
        registry_key.path, 'HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet')

    # Test a subkey of a virtual key.
    registry_key = win_registry.GetKeyByPath(
        'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Enum')
    self.assertIsNotNone(registry_key)

    self.assertEqual(
        registry_key.path,
        'HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Enum')

    # Test a non-existing key.
    registry_key = win_registry.GetKeyByPath(
        'HKEY_LOCAL_MACHINE\\System\\Bogus')
    self.assertIsNone(registry_key)

    # Tests Current value is not an integer.
    win_registry = TestWinRegistryKeyPathPrefixMismatch(
        registry_file_reader=TestREGFWinRegistryFileReaderMapped())

    with self.assertRaises(RuntimeError):
      win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System\\ControlSet001')

  def testGetRegistryFileMappingOnNTUserDat(self):
    """Tests the GetRegistryFileMapping function on a NTUSER.DAT file."""
    dat_test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(dat_test_path)

    log_test_path = self._GetTestFilePath(['NTUSER.DAT.LOG'])
    self._SkipIfPathNotExists(log_test_path)

    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(dat_test_path)

    try:
      key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
      self.assertEqual(key_path_prefix, 'HKEY_CURRENT_USER')

    finally:
      registry_file.Close()

    registry_file = win_registry._OpenFile(log_test_path)

    try:
      key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
      self.assertEqual(key_path_prefix, '')

    finally:
      registry_file.Close()

    key_path_prefix = win_registry.GetRegistryFileMapping(None)
    self.assertEqual(key_path_prefix, '')

  def testGetRegistryFileMappingOnSystem(self):
    """Tests the GetRegistryFileMapping function on a SYSTEM file."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)

    try:
      key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
      self.assertEqual(key_path_prefix, 'HKEY_LOCAL_MACHINE\\System')

    finally:
      registry_file.Close()

  # TODO: add GetRegistryFileMapping on UsrClass file test.

  # TODO: add tests for GetRootKey

  def testMapFile(self):
    """Tests the MapFile function."""
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)

    win_registry = registry.WinRegistry()
    key_path_prefix = win_registry.GetRegistryFileMapping(registry_file)
    win_registry.MapFile(key_path_prefix, registry_file)

  def testMapUserFile(self):
    """Tests the MapUserFile function."""
    test_path = self._GetTestFilePath(['NTUSER.DAT'])
    self._SkipIfPathNotExists(test_path)

    win_registry = registry.WinRegistry(
        registry_file_reader=TestREGFWinRegistryFileReader())

    registry_file = win_registry._OpenFile(test_path)

    win_registry = registry.WinRegistry()
    profile_path = '%SystemRoot%\\System32\\config\\systemprofile'
    win_registry.MapUserFile(profile_path, registry_file)

  # TODO: add tests for SplitKeyPath


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests for the Virtual Windows Registry key implementation."""

import unittest

from dfwinreg import registry
from dfwinreg import virtual

from tests import registry as test_registry
from tests import test_lib


class TestWinRegistry(object):
  """Windows Registry for testing."""

  # pylint: disable=redundant-returns-doc,unused-argument

  def GetKeyByPath(self, key_path):
    """Retrieves the key for a specific path.

    Args:
      key_path (str): Windows Registry key path.

    Returns:
      WinRegistryKey: Windows Registry key or None if not available.

    Raises:
      RuntimeError: if the root key is not supported.
    """
    return None


class ErrorWinRegistry(object):
  """Windows Registry for testing that fails."""

  # pylint: disable=redundant-returns-doc,unused-argument

  def GetKeyByPath(self, key_path):
    """Retrieves the key for a specific path.

    Args:
      key_path (str): Windows Registry key path.

    Returns:
      WinRegistryKey: Windows Registry key or None if not available.

    Raises:
      RuntimeError: if the root key is not supported.
    """
    raise RuntimeError('Not supported for testing')


class VirtualWinRegistryKeyTest(test_lib.BaseTestCase):
  """Tests for a virtual Windows Registry key."""

  # pylint: disable=protected-access

  def _CreateTestKey(self):
    """Creates a virtual Windows Registry key.

    Returns:
      VirtualWinRegistryKey: virtual Windows Registry key.
    """
    registry_key = virtual.VirtualWinRegistryKey(
        'HKEY_LOCAL_MACHINE', key_path='HKEY_LOCAL_MACHINE')

    sub_registry_key = virtual.VirtualWinRegistryKey('System')
    registry_key.AddSubkey(sub_registry_key.name, sub_registry_key)

    sub_registry_key = virtual.VirtualWinRegistryKey('Software')
    registry_key.AddSubkey(sub_registry_key.name, sub_registry_key)

    test_registry_key = virtual.VirtualWinRegistryKey('Classes')
    sub_registry_key.AddSubkey(test_registry_key.name, test_registry_key)

    return registry_key

  def _CreateTestRegistryWithMappedFile(self):
    """Creates a Windows Registry with a mapped file.

    Returns:
      WinRegistry: Windows Registry.
    """
    test_path = self._GetTestFilePath(['SYSTEM'])
    self._SkipIfPathNotExists(test_path)

    win_registry = registry.WinRegistry(
        registry_file_reader=test_registry.TestREGFWinRegistryFileReader())
    win_registry.OpenAndMapFile(test_path)

    return win_registry

  def testLastWrittenTime(self):
    """Tests the last_written_time property."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)
    self.assertIsNotNone(mapped_key.last_written_time)

  def testNumberOfSubkeys(self):
    """Tests the number_of_subkeys property."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)
    self.assertEqual(mapped_key.number_of_subkeys, 9)

  def testNumberOfValues(self):
    """Tests the number_of_values property."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath(
        'HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)
    self.assertEqual(mapped_key.number_of_values, 0)

    # Test virtual key fallback.
    registry_key = self._CreateTestKey()
    self.assertIsNotNone(registry_key)

    self.assertEqual(registry_key.number_of_values, 0)

  def testOffset(self):
    """Tests the offset property."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)
    self.assertEqual(mapped_key.offset, 4132)

  def testPropertiesWithMappedRegistry(self):
    """Tests the properties with a mapped registry."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)

    self.assertIsNone(mapped_key.class_name)
    self.assertIsNotNone(mapped_key.last_written_time)
    timestamp = mapped_key.last_written_time.timestamp
    self.assertEqual(timestamp, 131243722185199451)

    self.assertEqual(mapped_key.number_of_subkeys, 9)
    self.assertEqual(mapped_key.number_of_values, 0)
    self.assertEqual(mapped_key.offset, 4132)

  def testGetKeyFromRegistry(self):
    """Tests the _GetKeyFromRegistry function."""
    registry_key = virtual.VirtualWinRegistryKey(
        'HKEY_LOCAL_MACHINE', key_path='')
    registry_key._GetKeyFromRegistry()

    test_win_registry = TestWinRegistry()
    registry_key = virtual.VirtualWinRegistryKey(
        'HKEY_LOCAL_MACHINE', key_path='', registry=test_win_registry)
    registry_key._GetKeyFromRegistry()

    test_win_registry = ErrorWinRegistry()
    registry_key = virtual.VirtualWinRegistryKey(
        'HKEY_LOCAL_MACHINE', key_path='', registry=test_win_registry)
    registry_key._GetKeyFromRegistry()

  def testJoinKeyPath(self):
    """Tests the _JoinKeyPath function."""
    registry_key = virtual.VirtualWinRegistryKey(
        'HKEY_LOCAL_MACHINE', key_path='')

    expected_path = 'HKEY_LOCAL_MACHINE\\Software'
    path = registry_key._JoinKeyPath(['HKEY_LOCAL_MACHINE', 'Software'])
    self.assertEqual(path, expected_path)

  def testAddSubkey(self):
    """Tests the AddSubkey function."""
    registry_key = virtual.VirtualWinRegistryKey(
        'HKEY_LOCAL_MACHINE', key_path='')

    sub_registry_key = virtual.VirtualWinRegistryKey(
        'System', key_path='HKEY_LOCAL_MACHINE')

    registry_key.AddSubkey(sub_registry_key.name, sub_registry_key)

    with self.assertRaises(KeyError):
      registry_key.AddSubkey(sub_registry_key.name, sub_registry_key)

  def testGetSubkeyByIndex(self):
    """Tests the GetSubkeyByIndex function."""
    registry_key = self._CreateTestKey()

    sub_registry_key = registry_key.GetSubkeyByIndex(0)
    self.assertIsNotNone(sub_registry_key)

    expected_key_path = 'HKEY_LOCAL_MACHINE\\System'
    self.assertEqual(sub_registry_key.path, expected_key_path)

    with self.assertRaises(IndexError):
      registry_key.GetSubkeyByIndex(-1)

  def testGetSubkeyByIndexWithMappedRegistry(self):
    """Tests the GetSubkeyByIndex function with a mapped registry."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)

    sub_registry_key = mapped_key.GetSubkeyByIndex(0)
    self.assertIsNotNone(sub_registry_key)

  def testGetSubkeyByName(self):
    """Tests the GetSubkeyByName function."""
    registry_key = self._CreateTestKey()

    sub_registry_key = registry_key.GetSubkeyByName('Software')
    self.assertIsNotNone(sub_registry_key)

    expected_key_path = 'HKEY_LOCAL_MACHINE\\Software'
    self.assertEqual(sub_registry_key.path, expected_key_path)

    sub_registry_key = registry_key.GetSubkeyByName('Bogus')
    self.assertIsNone(sub_registry_key)

  def testGetSubkeyByNameWithMappedRegistry(self):
    """Tests the GetSubkeyByName function with a mapped registry."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)

    sub_registry_key = mapped_key.GetSubkeyByName('ControlSet001')
    self.assertIsNotNone(sub_registry_key)

  def testGetSubkeyByPath(self):
    """Tests the GetSubkeyByPath function."""
    registry_key = self._CreateTestKey()

    key_path = 'Software\\Classes'
    sub_registry_key = registry_key.GetSubkeyByPath(key_path)
    self.assertIsNotNone(sub_registry_key)
    self.assertEqual(sub_registry_key.name, 'Classes')

    expected_key_path = (
        'HKEY_LOCAL_MACHINE\\Software\\Classes')
    self.assertEqual(sub_registry_key.path, expected_key_path)

    key_path = 'Software\\Bogus'
    sub_registry_key = registry_key.GetSubkeyByPath(key_path)
    self.assertIsNone(sub_registry_key)

  def testGetSubkeyByPathWithMappedRegistry(self):
    """Tests the GetSubkeyByPath function with a mapped registry."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)

    sub_registry_key = mapped_key.GetSubkeyByPath('ControlSet001\\Control')
    self.assertIsNotNone(sub_registry_key)

  def testGetSubkeys(self):
    """Tests the GetSubkeys function."""
    registry_key = self._CreateTestKey()

    sub_registry_keys = list(registry_key.GetSubkeys())
    self.assertEqual(len(sub_registry_keys), 2)

  def testGetSubkeysWithMappedRegistry(self):
    """Tests the GetSubkeys function with a mapped registry."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)

    sub_registry_keys = list(mapped_key.GetSubkeys())
    self.assertEqual(len(sub_registry_keys), 9)

  def testGetValueByName(self):
    """Tests the GetValueByName function."""
    registry_key = self._CreateTestKey()

    registry_value = registry_key.GetValueByName('')
    self.assertIsNone(registry_value)

  def testGetValueByNameWithMappedRegistry(self):
    """Tests the GetValueByName function with a mapped registry."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)

    registry_value = mapped_key.GetValueByName('')
    self.assertIsNone(registry_value)

  def testGetValues(self):
    """Tests the GetValues function."""
    registry_key = self._CreateTestKey()

    values = list(registry_key.GetValues())
    self.assertEqual(len(values), 0)

  def testGetValuesWithMappedRegistry(self):
    """Tests the GetValues function with a mapped registry."""
    win_registry = self._CreateTestRegistryWithMappedFile()

    mapped_key = win_registry.GetKeyByPath('HKEY_LOCAL_MACHINE\\System')
    self.assertIsNotNone(mapped_key)

    values = list(mapped_key.GetValues())
    self.assertEqual(len(values), 0)


if __name__ == '__main__':
  unittest.main()
